package sun.security.provider.certpath;

import sun.security.x509.AuthorityInfoAccessExtension;
import java.security.cert.CertPathBuilderException;
import java.security.cert.PKIXCertPathBuilderResult;
import java.security.cert.CertPathBuilder;
import sun.security.x509.AccessDescription;
import java.security.cert.PKIXBuilderParameters;
import sun.security.x509.CRLDistributionPointsExtension;
import sun.security.x509.DistributionPoint;
import sun.security.x509.GeneralName;
import sun.security.x509.X500Name;
import sun.security.x509.GeneralNames;
import java.security.cert.CertificateException;
import sun.security.x509.X509CertImpl;
import java.util.Date;
import java.security.cert.X509CRLEntry;
import java.security.cert.CertificateRevokedException;
import sun.security.x509.PKIXExtensions;
import java.security.cert.CRLException;
import sun.security.x509.X509CRLEntryImpl;
import java.security.cert.CRLReason;
import java.security.cert.X509CRL;
import java.util.Arrays;
import java.security.cert.X509CRLSelector;
import java.util.HashSet;
import java.io.IOException;
import java.security.cert.Certificate;
import java.util.Collections;
import java.util.Collection;
import java.security.cert.CertStoreException;
import java.math.BigInteger;
import javax.security.auth.x500.X500Principal;
import java.security.cert.X509CertSelector;
import java.security.AccessController;
import java.security.Security;
import java.security.PrivilegedAction;
import java.net.URISyntaxException;
import java.util.Iterator;
import java.util.Set;
import java.security.NoSuchAlgorithmException;
import java.security.InvalidAlgorithmParameterException;
import java.security.cert.CollectionCertStoreParameters;
import java.util.ArrayList;
import java.security.PublicKey;
import java.security.cert.CertPathValidatorException;
import java.util.LinkedList;
import java.security.cert.Extension;
import java.util.Map;
import java.security.cert.CertStore;
import java.util.List;
import java.security.cert.X509Certificate;
import java.net.URI;
import java.security.cert.TrustAnchor;
import sun.security.util.Debug;
import java.security.cert.PKIXRevocationChecker;

class RevocationChecker extends PKIXRevocationChecker
{
    private static final Debug debug;
    private TrustAnchor anchor;
    private PKIX.ValidatorParams params;
    private boolean onlyEE;
    private boolean softFail;
    private boolean crlDP;
    private URI responderURI;
    private X509Certificate responderCert;
    private List<CertStore> certStores;
    private Map<X509Certificate, byte[]> ocspResponses;
    private List<Extension> ocspExtensions;
    private final boolean legacy;
    private LinkedList<CertPathValidatorException> softFailExceptions;
    private OCSPResponse.IssuerInfo issuerInfo;
    private PublicKey prevPubKey;
    private boolean crlSignFlag;
    private int certIndex;
    private RevocationChecker.Mode mode;
    private static final long MAX_CLOCK_SKEW = 900000L;
    private static final String HEX_DIGITS = "0123456789ABCDEFabcdef";
    private static final boolean[] ALL_REASONS;
    private static final boolean[] CRL_SIGN_USAGE;
    
    RevocationChecker() {
        this.softFailExceptions = new LinkedList();
        this.mode = RevocationChecker.Mode.PREFER_OCSP;
        this.legacy = false;
    }
    
    RevocationChecker(TrustAnchor trustAnchor, PKIX.ValidatorParams validatorParams) throws CertPathValidatorException {
        this.softFailExceptions = new LinkedList();
        this.mode = RevocationChecker.Mode.PREFER_OCSP;
        this.legacy = true;
        this.init(trustAnchor, validatorParams);
    }
    
    void init(TrustAnchor anchor, PKIX.ValidatorParams params) throws CertPathValidatorException {
        RevocationChecker.RevocationProperties revocationProperties = getRevocationProperties();
        URI ocspResponder = this.getOcspResponder();
        this.responderURI = ((ocspResponder == null) ? toURI(revocationProperties.ocspUrl) : ocspResponder);
        X509Certificate ocspResponderCert = this.getOcspResponderCert();
        this.responderCert = ((ocspResponderCert == null) ? getResponderCert(revocationProperties, params.trustAnchors(), params.certStores()) : ocspResponderCert);
        Set<PKIXRevocationChecker.Option> options = this.getOptions();
        for (PKIXRevocationChecker.Option obj : options) {
            switch (obj) {
                case ONLY_END_ENTITY:
                case PREFER_CRLS:
                case SOFT_FAIL:
                case NO_FALLBACK: {
                    continue;
                }
                default: {
                    throw new CertPathValidatorException("Unrecognized revocation parameter option: " + obj);
                }
            }
        }
        this.softFail = options.contains(PKIXRevocationChecker.Option.SOFT_FAIL);
        if (this.legacy) {
            this.mode = (revocationProperties.ocspEnabled ? RevocationChecker.Mode.PREFER_OCSP : RevocationChecker.Mode.ONLY_CRLS);
            this.onlyEE = revocationProperties.onlyEE;
        }
        else {
            if (options.contains(PKIXRevocationChecker.Option.NO_FALLBACK)) {
                if (options.contains(PKIXRevocationChecker.Option.PREFER_CRLS)) {
                    this.mode = RevocationChecker.Mode.ONLY_CRLS;
                }
                else {
                    this.mode = RevocationChecker.Mode.ONLY_OCSP;
                }
            }
            else if (options.contains(PKIXRevocationChecker.Option.PREFER_CRLS)) {
                this.mode = RevocationChecker.Mode.PREFER_CRLS;
            }
            this.onlyEE = options.contains(PKIXRevocationChecker.Option.ONLY_END_ENTITY);
        }
        if (this.legacy) {
            this.crlDP = revocationProperties.crlDPEnabled;
        }
        else {
            this.crlDP = true;
        }
        this.ocspResponses = this.getOcspResponses();
        this.ocspExtensions = this.getOcspExtensions();
        this.anchor = anchor;
        this.params = params;
        this.certStores = new ArrayList(params.certStores());
        try {
            this.certStores.add(CertStore.getInstance("Collection", new CollectionCertStoreParameters(params.certificates())));
        }
        catch (InvalidAlgorithmParameterException | NoSuchAlgorithmException obj2) {
            if (RevocationChecker.debug != null) {
                RevocationChecker.debug.println("RevocationChecker: error creating Collection CertStore: " + obj2);
            }
        }
    }
    
    private static URI toURI(String str) throws CertPathValidatorException {
        try {
            if (str != null) {
                return new URI(str);
            }
            return null;
        }
        catch (URISyntaxException cause) {
            throw new CertPathValidatorException("cannot parse ocsp.responderURL property", cause);
        }
    }
    
    private static RevocationChecker.RevocationProperties getRevocationProperties() {
        return (RevocationChecker.RevocationProperties)AccessController.doPrivileged(new PrivilegedAction<RevocationChecker.RevocationProperties>() {
            @Override
            public RevocationChecker.RevocationProperties run() {
                RevocationChecker.RevocationProperties revocationProperties = new RevocationChecker.RevocationProperties();
                String property = Security.getProperty("com.sun.security.onlyCheckRevocationOfEECert");
                revocationProperties.onlyEE = (property != null && property.equalsIgnoreCase("true"));
                String property2 = Security.getProperty("ocsp.enable");
                revocationProperties.ocspEnabled = (property2 != null && property2.equalsIgnoreCase("true"));
                revocationProperties.ocspUrl = Security.getProperty("ocsp.responderURL");
                revocationProperties.ocspSubject = Security.getProperty("ocsp.responderCertSubjectName");
                revocationProperties.ocspIssuer = Security.getProperty("ocsp.responderCertIssuerName");
                revocationProperties.ocspSerial = Security.getProperty("ocsp.responderCertSerialNumber");
                revocationProperties.crlDPEnabled = Boolean.getBoolean("com.sun.security.enableCRLDP");
                return revocationProperties;
            }
        });
    }
    
    private static X509Certificate getResponderCert(RevocationChecker.RevocationProperties revocationProperties, Set<TrustAnchor> set, List<CertStore> list) throws CertPathValidatorException {
        if (revocationProperties.ocspSubject != null) {
            return getResponderCert(revocationProperties.ocspSubject, set, list);
        }
        if (revocationProperties.ocspIssuer != null && revocationProperties.ocspSerial != null) {
            return getResponderCert(revocationProperties.ocspIssuer, revocationProperties.ocspSerial, set, list);
        }
        if (revocationProperties.ocspIssuer != null || revocationProperties.ocspSerial != null) {
            throw new CertPathValidatorException("Must specify both ocsp.responderCertIssuerName and ocsp.responderCertSerialNumber properties");
        }
        return null;
    }
    
    private static X509Certificate getResponderCert(String name, Set<TrustAnchor> set, List<CertStore> list) throws CertPathValidatorException {
        X509CertSelector x509CertSelector = new X509CertSelector();
        try {
            x509CertSelector.setSubject(new X500Principal(name));
        }
        catch (IllegalArgumentException cause) {
            throw new CertPathValidatorException("cannot parse ocsp.responderCertSubjectName property", cause);
        }
        return getResponderCert(x509CertSelector, set, list);
    }
    
    private static X509Certificate getResponderCert(String name, String s, Set<TrustAnchor> set, List<CertStore> list) throws CertPathValidatorException {
        X509CertSelector x509CertSelector = new X509CertSelector();
        try {
            x509CertSelector.setIssuer(new X500Principal(name));
        }
        catch (IllegalArgumentException cause) {
            throw new CertPathValidatorException("cannot parse ocsp.responderCertIssuerName property", cause);
        }
        try {
            x509CertSelector.setSerialNumber(new BigInteger(stripOutSeparators(s), 16));
        }
        catch (NumberFormatException cause2) {
            throw new CertPathValidatorException("cannot parse ocsp.responderCertSerialNumber property", cause2);
        }
        return getResponderCert(x509CertSelector, set, list);
    }
    
    private static X509Certificate getResponderCert(X509CertSelector selector, Set<TrustAnchor> set, List<CertStore> list) throws CertPathValidatorException {
        Iterator<Object> iterator = set.iterator();
        while (iterator.hasNext()) {
            X509Certificate trustedCert = ((TrustAnchor)iterator.next()).getTrustedCert();
            if (trustedCert == null) {
                continue;
            }
            if (selector.match(trustedCert)) {
                return trustedCert;
            }
            continue;
        }
        for (CertStore certStore : list) {
            try {
                Collection<Object> certificates = certStore.getCertificates(selector);
                if (!certificates.isEmpty()) {
                    return (X509Certificate)certificates.iterator().next();
                }
                continue;
            }
            catch (CertStoreException obj) {
                if (RevocationChecker.debug != null) {
                    RevocationChecker.debug.println("CertStore exception:" + obj);
                }
                else {
                    continue;
                }
            }
        }
        throw new CertPathValidatorException("Cannot find the responder's certificate (set using the OCSP security properties).");
    }
    
    @Override
    public void init(boolean b) throws CertPathValidatorException {
        if (b) {
            throw new CertPathValidatorException("forward checking not supported");
        }
        if (this.anchor != null) {
            this.issuerInfo = new OCSPResponse.IssuerInfo(this.anchor);
            this.prevPubKey = this.issuerInfo.getPublicKey();
        }
        this.crlSignFlag = true;
        if (this.params != null && this.params.certPath() != null) {
            this.certIndex = this.params.certPath().getCertificates().size() - 1;
        }
        else {
            this.certIndex = -1;
        }
        this.softFailExceptions.clear();
        return;
    }
    
    @Override
    public boolean isForwardCheckingSupported() {
        return false;
    }
    
    @Override
    public Set<String> getSupportedExtensions() {
        return null;
    }
    
    @Override
    public List<CertPathValidatorException> getSoftFailExceptions() {
        return Collections.unmodifiableList(this.softFailExceptions);
    }
    
    @Override
    public void check(Certificate certificate, Collection<String> collection) throws CertPathValidatorException {
        this.check((X509Certificate)certificate, collection, this.prevPubKey, this.crlSignFlag);
    }
    
    private void check(X509Certificate x509Certificate, Collection<String> collection, PublicKey publicKey, boolean b) throws CertPathValidatorException {
        if (RevocationChecker.debug != null) {
            RevocationChecker.debug.println("RevocationChecker.check: checking cert\n  SN: " + Debug.toHexString(x509Certificate.getSerialNumber()) + "\n  Subject: " + x509Certificate.getSubjectX500Principal() + "\n  Issuer: " + x509Certificate.getIssuerX500Principal());
        }
        try {
            if (this.onlyEE && x509Certificate.getBasicConstraints() != -1) {
                if (RevocationChecker.debug != null) {
                    RevocationChecker.debug.println("Skipping revocation check; cert is not an end entity cert");
                }
                return;
            }
            else {
                switch (this.mode) {
                    case PREFER_OCSP:
                    case ONLY_OCSP: {
                        this.checkOCSP(x509Certificate, collection);
                        break;
                    }
                    case PREFER_CRLS:
                    case ONLY_CRLS: {
                        this.checkCRLs(x509Certificate, collection, null, publicKey, b);
                        break;
                    }
                }
            }
        }
        catch (CertPathValidatorException ex) {
            if (ex.getReason() == CertPathValidatorException.BasicReason.REVOKED) {
                throw ex;
            }
            int softFailException = this.isSoftFailException(ex);
            if (!(softFailException == 0)) {
                if (this.mode == RevocationChecker.Mode.ONLY_OCSP || this.mode == RevocationChecker.Mode.ONLY_CRLS) {
                    this.updateState(x509Certificate);
                    return;
                }
            }
            else if (this.mode == RevocationChecker.Mode.ONLY_OCSP || this.mode == RevocationChecker.Mode.ONLY_CRLS) {
                throw ex;
            }
            if (RevocationChecker.debug != null) {
                RevocationChecker.debug.println("RevocationChecker.check() " + ex.getMessage());
                RevocationChecker.debug.println("RevocationChecker.check() preparing to failover");
            }
            try {
                switch (this.mode) {
                    case PREFER_OCSP: {
                        this.checkCRLs(x509Certificate, collection, null, publicKey, b);
                        break;
                    }
                    case PREFER_CRLS: {
                        this.checkOCSP(x509Certificate, collection);
                        break;
                    }
                }
            }
            catch (CertPathValidatorException exception) {
                if (RevocationChecker.debug != null) {
                    RevocationChecker.debug.println("RevocationChecker.check() failover failed");
                    RevocationChecker.debug.println("RevocationChecker.check() " + exception.getMessage());
                }
                if (exception.getReason() == CertPathValidatorException.BasicReason.REVOKED) {
                    throw exception;
                }
                if (!this.isSoftFailException(exception)) {
                    ex.addSuppressed(exception);
                    throw ex;
                }
                if (softFailException == 0) {
                    throw ex;
                }
            }
            this.updateState(x509Certificate);
        }
        finally {
            this.updateState(x509Certificate);
        }
    }
    
    private boolean isSoftFailException(CertPathValidatorException ex) {
        if (this.softFail && ex.getReason() == CertPathValidatorException.BasicReason.UNDETERMINED_REVOCATION_STATUS) {
            this.softFailExceptions.addFirst(new CertPathValidatorException(ex.getMessage(), ex.getCause(), this.params.certPath(), this.certIndex, ex.getReason()));
            return true;
        }
        return false;
    }
    
    private void updateState(X509Certificate x509Certificate) throws CertPathValidatorException {
        this.issuerInfo = new OCSPResponse.IssuerInfo(this.anchor, x509Certificate);
        PublicKey prevPubKey = x509Certificate.getPublicKey();
        if (PKIX.isDSAPublicKeyWithoutParams(prevPubKey)) {
            prevPubKey = BasicChecker.makeInheritedParamsKey(prevPubKey, this.prevPubKey);
        }
        this.prevPubKey = prevPubKey;
        this.crlSignFlag = certCanSignCrl(x509Certificate);
        if (this.certIndex > 0) {
            --this.certIndex;
        }
    }
    
    private void checkCRLs(X509Certificate x509Certificate, Collection<String> collection, Set<X509Certificate> set, PublicKey publicKey, boolean b) throws CertPathValidatorException {
        this.checkCRLs(x509Certificate, publicKey, null, b, true, set, this.params.trustAnchors());
    }
    
    static boolean isCausedByNetworkIssue(String s, CertStoreException ex) {
        Throwable cause = ex.getCause();
        int n = -1;
        switch (s.hashCode()) {
            case 2331559: {
                if (s.equals("LDAP")) {
                    n = 0;
                    break;
                }
                break;
            }
            case 133315663: {
                if (s.equals("SSLServer")) {
                    n = 1;
                    break;
                }
                break;
            }
            case 84300: {
                if (s.equals("URI")) {
                    n = 2;
                    break;
                }
                break;
            }
        }
        int n2;
        switch (n) {
            case 0: {
                if (cause != null) {
                    String name = cause.getClass().getName();
                    n2 = (name.equals("javax.naming.ServiceUnavailableException") || name.equals("javax.naming.CommunicationException"));
                    break;
                }
                n2 = false;
                break;
            }
            case 1: {
                n2 = (cause != null && cause instanceof IOException);
                break;
            }
            case 2: {
                n2 = (cause != null && cause instanceof IOException);
                break;
            }
            default: {
                return false;
            }
        }
        return n2;
    }
    
    private void checkCRLs(X509Certificate certificateChecking, PublicKey publicKey, X509Certificate x509Certificate, boolean b, boolean b2, Set<X509Certificate> set, Set<TrustAnchor> set2) throws CertPathValidatorException {
        if (RevocationChecker.debug != null) {
            RevocationChecker.debug.println("RevocationChecker.checkCRLs() ---checking revocation status ...");
        }
        if (set != null && set.contains(certificateChecking)) {
            if (RevocationChecker.debug != null) {
                RevocationChecker.debug.println("RevocationChecker.checkCRLs() circular dependency");
            }
            throw new CertPathValidatorException("Could not determine revocation status", null, null, -1, CertPathValidatorException.BasicReason.UNDETERMINED_REVOCATION_STATUS);
        }
        else {
            HashSet set3 = new HashSet();
            HashSet<X509CRL> set4 = new HashSet();
            X509CRLSelector selector = new X509CRLSelector();
            selector.setCertificateChecking(certificateChecking);
            CertPathHelper.setDateAndTime(selector, this.params.date(), 900000L);
            Object o = null;
            for (CertStore certStore : this.certStores) {
                try {
                    Iterator<Object> iterator2 = certStore.getCRLs(selector).iterator();
                    while (iterator2.hasNext()) {
                        set3.add((E)(E)iterator2.next());
                    }
                }
                catch (CertStoreException cause) {
                    if (RevocationChecker.debug != null) {
                        RevocationChecker.debug.println("RevocationChecker.checkCRLs() CertStoreException: " + cause.getMessage());
                    }
                    if (o == null && isCausedByNetworkIssue(certStore.getType(), cause)) {
                        o = new CertPathValidatorException("Unable to determine revocation status due to network error", cause, null, -1, CertPathValidatorException.BasicReason.UNDETERMINED_REVOCATION_STATUS);
                    }
                    else {
                        continue;
                    }
                }
            }
            if (RevocationChecker.debug != null) {
                RevocationChecker.debug.println("RevocationChecker.checkCRLs() possible crls.size() = " + set3.size());
            }
            boolean[] array = new boolean[9];
            if (!set3.isEmpty()) {
                set4.addAll(this.verifyPossibleCRLs(set3, certificateChecking, publicKey, b, array, set2));
            }
            if (RevocationChecker.debug != null) {
                RevocationChecker.debug.println("RevocationChecker.checkCRLs() approved crls.size() = " + set4.size());
            }
            if (!set4.isEmpty() && Arrays.equals(array, RevocationChecker.ALL_REASONS)) {
                this.checkApprovedCRLs(certificateChecking, set4);
            }
            else {
                try {
                    if (this.crlDP) {
                        set4.addAll(DistributionPointFetcher.getCRLs(selector, b, publicKey, x509Certificate, this.params.sigProvider(), this.certStores, array, set2, null, this.params.variant()));
                    }
                }
                catch (CertStoreException ex) {
                    if (ex instanceof PKIX.CertStoreTypeException && isCausedByNetworkIssue(((PKIX.CertStoreTypeException)ex).getType(), ex)) {
                        throw new CertPathValidatorException("Unable to determine revocation status due to network error", ex, null, -1, CertPathValidatorException.BasicReason.UNDETERMINED_REVOCATION_STATUS);
                    }
                    throw new CertPathValidatorException(ex);
                }
                if (!set4.isEmpty() && Arrays.equals(array, RevocationChecker.ALL_REASONS)) {
                    this.checkApprovedCRLs(certificateChecking, set4);
                }
                else if (b2) {
                    try {
                        this.verifyWithSeparateSigningKey(certificateChecking, publicKey, b, set);
                        return;
                    }
                    catch (CertPathValidatorException ex2) {
                        if (o != null) {
                            throw o;
                        }
                        throw ex2;
                    }
                }
                else {
                    if (o != null) {
                        throw o;
                    }
                    throw new CertPathValidatorException("Could not determine revocation status", null, null, -1, CertPathValidatorException.BasicReason.UNDETERMINED_REVOCATION_STATUS);
                }
            }
            return;
        }
    }
    
    private void checkApprovedCRLs(X509Certificate certificate, Set<X509CRL> set) throws CertPathValidatorException {
        if (RevocationChecker.debug != null) {
            BigInteger serialNumber = certificate.getSerialNumber();
            RevocationChecker.debug.println("RevocationChecker.checkApprovedCRLs() starting the final sweep...");
            RevocationChecker.debug.println("RevocationChecker.checkApprovedCRLs() cert SN: " + serialNumber.toString());
        }
        CRLReason unspecified = CRLReason.UNSPECIFIED;
        for (X509CRL x509CRL : set) {
            X509CRLEntry revokedCertificate = x509CRL.getRevokedCertificate(certificate);
            if (revokedCertificate != null) {
                X509CRLEntryImpl impl;
                try {
                    impl = X509CRLEntryImpl.toImpl(revokedCertificate);
                }
                catch (CRLException cause) {
                    throw new CertPathValidatorException(cause);
                }
                if (RevocationChecker.debug != null) {
                    RevocationChecker.debug.println("RevocationChecker.checkApprovedCRLs() CRL entry: " + impl.toString());
                }
                Set<String> criticalExtensionOIDs = impl.getCriticalExtensionOIDs();
                if (criticalExtensionOIDs != null && !criticalExtensionOIDs.isEmpty()) {
                    criticalExtensionOIDs.remove(PKIXExtensions.ReasonCode_Id.toString());
                    criticalExtensionOIDs.remove(PKIXExtensions.CertificateIssuer_Id.toString());
                    if (!criticalExtensionOIDs.isEmpty()) {
                        throw new CertPathValidatorException("Unrecognized critical extension(s) in revoked CRL entry");
                    }
                }
                CRLReason reason = impl.getRevocationReason();
                if (reason == null) {
                    reason = CRLReason.UNSPECIFIED;
                }
                Date revocationDate = impl.getRevocationDate();
                if (revocationDate.before(this.params.date())) {
                    CertificateRevokedException cause2 = new CertificateRevokedException(revocationDate, reason, x509CRL.getIssuerX500Principal(), impl.getExtensions());
                    throw new CertPathValidatorException(cause2.getMessage(), cause2, null, -1, CertPathValidatorException.BasicReason.REVOKED);
                }
                continue;
            }
            else {
                continue;
            }
        }
    }
    
    private void checkOCSP(X509Certificate cert, Collection<String> collection) throws CertPathValidatorException {
        X509CertImpl impl;
        try {
            impl = X509CertImpl.toImpl(cert);
        }
        catch (CertificateException cause) {
            throw new CertPathValidatorException(cause);
        }
        CertId certId;
        OCSPResponse check;
        try {
            certId = new CertId(this.issuerInfo.getName(), this.issuerInfo.getPublicKey(), impl.getSerialNumberObject());
            byte[] array = (byte[])this.ocspResponses.get(cert);
            if (array != null) {
                if (RevocationChecker.debug != null) {
                    RevocationChecker.debug.println("Found cached OCSP response");
                }
                check = new OCSPResponse(array);
                byte[] value = null;
                for (Extension extension : this.ocspExtensions) {
                    if (extension.getId().equals("1.3.6.1.5.5.7.48.1.2")) {
                        value = extension.getValue();
                    }
                    else {
                        continue;
                    }
                }
                check.verify(Collections.singletonList(certId), this.issuerInfo, this.responderCert, this.params.date(), value, this.params.variant());
            }
            else {
                URI uri = (this.responderURI != null) ? this.responderURI : OCSP.getResponderURI(impl);
                if (uri == null) {
                    throw new CertPathValidatorException("Certificate does not specify OCSP responder", null, null, -1);
                }
                check = OCSP.check(Collections.singletonList(certId), uri, this.issuerInfo, this.responderCert, null, this.ocspExtensions, this.params.variant());
            }
        }
        catch (IOException cause2) {
            throw new CertPathValidatorException("Unable to determine revocation status due to network error", cause2, null, -1, CertPathValidatorException.BasicReason.UNDETERMINED_REVOCATION_STATUS);
        }
        OCSPResponse.SingleResponse singleResponse = check.getSingleResponse(certId);
        OCSP.RevocationStatus.CertStatus certStatus = singleResponse.getCertStatus();
        if (certStatus == OCSP.RevocationStatus.CertStatus.REVOKED) {
            Date revocationTime = singleResponse.getRevocationTime();
            if (revocationTime.before(this.params.date())) {
                CertificateRevokedException cause3 = new CertificateRevokedException(revocationTime, singleResponse.getRevocationReason(), check.getSignerCertificate().getSubjectX500Principal(), singleResponse.getSingleExtensions());
                throw new CertPathValidatorException(cause3.getMessage(), cause3, null, -1, CertPathValidatorException.BasicReason.REVOKED);
            }
        }
        else if (certStatus == OCSP.RevocationStatus.CertStatus.UNKNOWN) {
            throw new CertPathValidatorException("Certificate's revocation status is unknown", null, this.params.certPath(), -1, CertPathValidatorException.BasicReason.UNDETERMINED_REVOCATION_STATUS);
        }
    }
    
    private static String stripOutSeparators(String s) {
        char[] charArray = s.toCharArray();
        StringBuilder sb = new StringBuilder();
        for (int i = 0; i < charArray.length; ++i) {
            if ("0123456789ABCDEFabcdef".indexOf((int)charArray[i]) != -1) {
                sb.append(charArray[i]);
            }
        }
        return sb.toString();
    }
    
    static boolean certCanSignCrl(X509Certificate x509Certificate) {
        boolean[] keyUsage = x509Certificate.getKeyUsage();
        return keyUsage != null && keyUsage[6];
    }
    
    private Collection<X509CRL> verifyPossibleCRLs(Set<X509CRL> set, X509Certificate cert, PublicKey publicKey, boolean b, boolean[] a, Set<TrustAnchor> set2) throws CertPathValidatorException {
        try {
            X509CertImpl impl = X509CertImpl.toImpl(cert);
            if (RevocationChecker.debug != null) {
                RevocationChecker.debug.println("RevocationChecker.verifyPossibleCRLs: Checking CRLDPs for " + impl.getSubjectX500Principal());
            }
            CRLDistributionPointsExtension crlDistributionPointsExtension = impl.getCRLDistributionPointsExtension();
            List<DistributionPoint> list;
            if (crlDistributionPointsExtension == null) {
                list = Collections.singletonList(new DistributionPoint(new GeneralNames().add(new GeneralName((X500Name)impl.getIssuerDN())), null, null));
            }
            else {
                list = crlDistributionPointsExtension.get("points");
            }
            HashSet o = new HashSet();
            for (DistributionPoint distributionPoint : list) {
                for (X509CRL x509CRL : set) {
                    if (DistributionPointFetcher.verifyCRL(impl, distributionPoint, x509CRL, a, b, publicKey, null, this.params.sigProvider(), set2, this.certStores, this.params.date(), this.params.variant())) {
                        o.add(x509CRL);
                    }
                    else {
                        continue;
                    }
                }
                if (Arrays.equals(a, RevocationChecker.ALL_REASONS)) {
                    break;
                }
                continue;
            }
            return o;
        }
        catch (CertificateException | CRLException | IOException ex) {
            if (RevocationChecker.debug != null) {
                RevocationChecker.debug.println("Exception while verifying CRL: " + ex.getMessage());
                ex.printStackTrace();
            }
            return Collections.emptySet();
        }
    }
    
    private void verifyWithSeparateSigningKey(X509Certificate x509Certificate, PublicKey publicKey, boolean b, Set<X509Certificate> set) throws CertPathValidatorException {
        String str = "revocation status";
        if (RevocationChecker.debug != null) {
            RevocationChecker.debug.println("RevocationChecker.verifyWithSeparateSigningKey() ---checking " + str + "...");
        }
        if (set != null && set.contains(x509Certificate)) {
            if (RevocationChecker.debug != null) {
                RevocationChecker.debug.println("RevocationChecker.verifyWithSeparateSigningKey() circular dependency");
            }
            throw new CertPathValidatorException("Could not determine revocation status", null, null, -1, CertPathValidatorException.BasicReason.UNDETERMINED_REVOCATION_STATUS);
        }
        else {
            if (!b) {
                this.buildToNewKey(x509Certificate, null, set);
            }
            else {
                this.buildToNewKey(x509Certificate, publicKey, set);
            }
            return;
        }
    }
    
    private void buildToNewKey(X509Certificate cert, PublicKey publicKey, Set<X509Certificate> set) throws CertPathValidatorException {
        if (RevocationChecker.debug != null) {
            RevocationChecker.debug.println("RevocationChecker.buildToNewKey() starting work");
        }
        HashSet<PublicKey> set2 = new HashSet();
        if (publicKey != null) {
            set2.add(publicKey);
        }
        RevocationChecker.RejectKeySelector targetConstraints = new RevocationChecker.RejectKeySelector(set2);
        targetConstraints.setSubject(cert.getIssuerX500Principal());
        targetConstraints.setKeyUsage(RevocationChecker.CRL_SIGN_USAGE);
        Set<TrustAnchor> trustAnchors = (this.anchor == null) ? this.params.trustAnchors() : Collections.singleton(this.anchor);
        PKIXBuilderParameters params;
        try {
            params = new PKIXBuilderParameters(trustAnchors, targetConstraints);
        }
        catch (InvalidAlgorithmParameterException cause) {
            throw new RuntimeException(cause);
        }
        params.setInitialPolicies(this.params.initialPolicies());
        params.setCertStores(this.certStores);
        params.setExplicitPolicyRequired(this.params.explicitPolicyRequired());
        params.setPolicyMappingInhibited(this.params.policyMappingInhibited());
        params.setAnyPolicyInhibited(this.params.anyPolicyInhibited());
        params.setDate(this.params.date());
        params.setCertPathCheckers(this.params.certPathCheckers());
        params.setSigProvider(this.params.sigProvider());
        params.setRevocationEnabled(false);
        if (Builder.USE_AIA) {
            X509CertImpl impl = null;
            try {
                impl = X509CertImpl.toImpl(cert);
            }
            catch (CertificateException obj) {
                if (RevocationChecker.debug != null) {
                    RevocationChecker.debug.println("RevocationChecker.buildToNewKey: error decoding cert: " + obj);
                }
            }
            AuthorityInfoAccessExtension authorityInfoAccessExtension = null;
            if (impl != null) {
                authorityInfoAccessExtension = impl.getAuthorityInfoAccessExtension();
            }
            if (authorityInfoAccessExtension != null) {
                List<AccessDescription> accessDescriptions = authorityInfoAccessExtension.getAccessDescriptions();
                if (accessDescriptions != null) {
                    Iterator<Object> iterator = accessDescriptions.iterator();
                    while (iterator.hasNext()) {
                        CertStore instance = URICertStore.getInstance((AccessDescription)iterator.next());
                        if (instance != null) {
                            if (RevocationChecker.debug != null) {
                                RevocationChecker.debug.println("adding AIAext CertStore");
                            }
                            params.addCertStore(instance);
                        }
                        else {
                            continue;
                        }
                    }
                }
            }
        }
        Label_0398: {
            CertPathBuilder instance2;
            try {
                instance2 = CertPathBuilder.getInstance("PKIX");
                break Label_0398;
            }
            catch (NoSuchAlgorithmException cause2) {
                throw new CertPathValidatorException(cause2);
            }
            try {
                while (true) {
                    if (RevocationChecker.debug != null) {
                        RevocationChecker.debug.println("RevocationChecker.buildToNewKey() about to try build ...");
                    }
                    PKIXCertPathBuilderResult pkixCertPathBuilderResult = (PKIXCertPathBuilderResult)instance2.build(params);
                    if (RevocationChecker.debug != null) {
                        RevocationChecker.debug.println("RevocationChecker.buildToNewKey() about to check revocation ...");
                    }
                    if (set == null) {
                        set = new HashSet();
                    }
                    set.add(cert);
                    TrustAnchor trustAnchor = pkixCertPathBuilderResult.getTrustAnchor();
                    PublicKey publicKey2 = trustAnchor.getCAPublicKey();
                    if (publicKey2 == null) {
                        publicKey2 = trustAnchor.getTrustedCert().getPublicKey();
                    }
                    boolean certCanSignCrl = true;
                    List<Object> certificates = pkixCertPathBuilderResult.getCertPath().getCertificates();
                    try {
                        X509Certificate obj2;
                        for (int i = certificates.size() - 1; i >= 0; certCanSignCrl = certCanSignCrl(obj2), publicKey2 = obj2.getPublicKey(), --i) {
                            obj2 = (X509Certificate)certificates.get(i);
                            if (RevocationChecker.debug != null) {
                                RevocationChecker.debug.println("RevocationChecker.buildToNewKey() index " + i + " checking " + obj2);
                            }
                            this.checkCRLs(obj2, publicKey2, null, (boolean)certCanSignCrl, true, set, trustAnchors);
                        }
                    }
                    catch (CertPathValidatorException ex) {
                        set2.add(pkixCertPathBuilderResult.getPublicKey());
                        continue;
                    }
                    if (RevocationChecker.debug != null) {
                        RevocationChecker.debug.println("RevocationChecker.buildToNewKey() got key " + pkixCertPathBuilderResult.getPublicKey());
                    }
                    PublicKey publicKey3 = pkixCertPathBuilderResult.getPublicKey();
                    X509Certificate x509Certificate = certificates.isEmpty() ? null : ((X509Certificate)certificates.get(0));
                    try {
                        this.checkCRLs(cert, publicKey3, x509Certificate, true, false, null, this.params.trustAnchors());
                        return;
                    }
                    catch (CertPathValidatorException ex2) {
                        if (ex2.getReason() == CertPathValidatorException.BasicReason.REVOKED) {
                            throw ex2;
                        }
                        set2.add(publicKey3);
                        continue;
                    }
                }
            }
            catch (InvalidAlgorithmParameterException cause3) {
                throw new CertPathValidatorException(cause3);
            }
            catch (CertPathBuilderException ex3) {
                throw new CertPathValidatorException("Could not determine revocation status", null, null, -1, CertPathValidatorException.BasicReason.UNDETERMINED_REVOCATION_STATUS);
            }
        }
    }
    
    static {
        debug = Debug.getInstance("certpath");
        ALL_REASONS = new boolean[] { true, true, true, true, true, true, true, true, true };
        CRL_SIGN_USAGE = new boolean[] { false, false, false, false, false, false, true };
    }
    
    private enum Mode
    {
        PREFER_OCSP, 
        PREFER_CRLS, 
        ONLY_CRLS, 
        ONLY_OCSP;
    }
    
    private static class RevocationProperties
    {
        boolean onlyEE;
        boolean ocspEnabled;
        boolean crlDPEnabled;
        String ocspUrl;
        String ocspSubject;
        String ocspIssuer;
        String ocspSerial;
    }
    
    private static class RejectKeySelector extends X509CertSelector
    {
        private final Set<PublicKey> badKeySet;
        
        RejectKeySelector(Set<PublicKey> badKeySet) {
            this.badKeySet = badKeySet;
        }
        
        @Override
        public boolean match(Certificate cert) {
            if (!super.match(cert)) {
                return false;
            }
            if (this.badKeySet.contains(cert.getPublicKey())) {
                if (RevocationChecker.debug != null) {
                    RevocationChecker.debug.println("RejectKeySelector.match: bad key");
                }
                return false;
            }
            else {
                if (RevocationChecker.debug != null) {
                    RevocationChecker.debug.println("RejectKeySelector.match: returning true");
                }
                return true;
            }
        }
        
        @Override
        public String toString() {
            StringBuilder sb = new StringBuilder();
            sb.append("RejectKeySelector: [\n");
            sb.append(super.toString());
            sb.append(this.badKeySet);
            sb.append("]");
            return sb.toString();
        }
    }
}
